@@ -108,7 +108,21 @@ class KitsuAPIWrapper(flow.Object):
             return None
         else:
             return data['id']
-    
+
+    def get_project_fps(self):
+        import requests
+        try:
+            data = gazu.project.get_project_by_name(
+                self._config.project_name.get()
+            )
+        except requests.exceptions.ConnectionError:
+            return None
+        else:
+            return data['fps']
+
+    def get_entity_type_data(self, entity_type_id):
+        return gazu.entity.get_entity_type(entity_type_id)
+   
     def get_shot_data(self, name, sequence):
         if isinstance(sequence, str):
             sequence = self.get_sequence_data(sequence)
@@ -210,7 +224,10 @@ class KitsuAPIWrapper(flow.Object):
             name
         )
 
-    def get_assets_data(self, asset_type):
+    def get_assets_data(self, asset_type=None):
+        if not asset_type:
+            return gazu.asset.all_assets_for_project(self._config.project_id.get())
+
         if isinstance(asset_type, str):
             asset_type = self.get_asset_type_data(asset_type)
 
@@ -273,10 +290,10 @@ class KitsuAPIWrapper(flow.Object):
         return task
     
     def get_assign_tasks(self):
-        return gazu.user.all_tasks_to_do()
+        return [task for task in gazu.user.all_tasks_to_do() if task['project_name'] == self._config.project_name.get()]
 
     def get_done_tasks(self):
-        return gazu.user.all_done_tasks()
+        return [task for task in gazu.user.all_done_tasks() if task['project_name'] == self._config.project_name.get()]
 
     def get_all_comments_for_task(self, task_id):
         return gazu.task.all_comments_for_task(task_id)
@@ -372,6 +389,21 @@ class KitsuAPIWrapper(flow.Object):
             return None
         
         return kitsu_id in assignees
+
+    def get_task_status(self, name=None, short_name=None):
+        if name:
+            return gazu.task.get_task_status_by_name(name)
+        if short_name:
+            return gazu.task.get_task_status_by_short_name(short_name)
+
+    def set_task_status(self, task_id, task_status_name, comment='', files=None):
+        status = gazu.task.get_task_status_by_name(task_status_name)
+
+        if status is not None:
+            comment = gazu.task.add_comment(task_id, status, comment=comment)
+
+            if files:
+                gazu.task.add_attachment_files_to_comment(task_id, comment, files)
     
     def set_shot_task_status(self, sequence_name, shot_name, task_type_name, task_status_name, comment=''):
         task = self.get_shot_task(sequence_name, shot_name, task_type_name)
