@@ -20,16 +20,32 @@ from kabaret.subprocess_manager.flow import RunAction
 from kabaret.flow_entities.entities import EntityCollection, Entity, Property, PropertyValue
 
 from ..utils.os import zip_folder, unzip_archive
+from ..utils.kabaret.flow_entities.entities import CustomEntity, CustomEntityCollection, PropertyChoiceValue
 
 from .dependency import get_dependencies
 from .maputils import ItemMap, CreateGenericAction, ClearMapAction, RemoveGenericAction, SimpleCreateAction
 from .runners import Runner, PythonRunner
 
 
-class StaticSiteTypeChoices(flow.values.ChoiceValue):
+class SiteTypeChoiceValue(PropertyChoiceValue):
+
     CHOICES = ['Studio', 'User']
 
 
+class SiteTypeSessionChoiceValue(flow.values.SessionValue):
+
+    CHOICES = ['Studio', 'User']
+
+    def choices(self):
+        return self.__class__.CHOICES
+
+
+class ExchangeSiteChoiceValue(flow.values.ChoiceValue):
+
+    def choices(self):
+        return self.root().project().get_exchange_sites().mapped_names()
+
+
 class StaticSiteSyncStatusChoices(flow.values.ChoiceValue):
     CHOICES = ['NotAvailable', 'Requested', 'Available']
 
@@ -637,39 +653,6 @@ class ActiveSiteChoiceValue(flow.values.SessionValue):
         return self._choices.get()
 
 
-class ActiveSiteAutoSelectChoiceValue(ActiveSiteChoiceValue):
-    
-    def choices(self):
-        choices = super(ActiveSiteAutoSelectChoiceValue, self).choices()
-        choices = ['Auto select'] + choices.copy()
-        
-        return choices
-
-
-class ActiveSitesMultichoiceValue(ActiveSiteChoiceValue):
-    
-    DEFAULT_EDITOR = 'multichoice'
-    
-    exclude_choice = flow.SessionParam()
-    
-    def choices(self):
-        if self._choices.get() is None:
-            working_sites = self.root().project().get_working_sites()
-            names = working_sites.get_site_names(use_custom_order=True, active_only=True)
-            self._choices.set(names)
-        
-        choices = self._choices.get().copy()
-        exclude_choice = self.exclude_choice.get()
-        
-        if exclude_choice is not None:
-            try:
-                choices.remove(exclude_choice)
-            except ValueError:
-                pass
-        
-        return choices
-
-
 class ActiveSiteRevisionAvailableChoiceValue(flow.values.ChoiceValue):
     
     DEFAULT_EDITOR = 'choice'
@@ -882,67 +865,6 @@ class Request(RequestAs):
         )
 
 
-class SyncErrorCounter(flow.DynamicMap):
-
-    _error_count = flow.Param([])
-
-    def refresh_error_count(self):
-        count = []
-        sites = self.root().project().get_working_sites()
-        site_names = sites.get_site_names(use_custom_order=True, active_only=True)
-
-        for name in site_names:
-            s = sites[name]
-            count.append((s.name(), s.oid(), len(s.get_jobs(status='ERROR'))))
-        
-        self._error_count.set(count)
-    
-    def columns(self):
-        return ['Site', 'Nb errors']
-
-    def rows(self):
-        count = self._error_count.get()
-        rows = []
-
-        for site_name, site_oid, nb_err in count:
-            style = {'icon': ('icons.libreflow', 'blank')}
-            oid = site_oid + '/queue'
-            
-            if nb_err > 0:
-                style['foreground-color'] = '#D5000D'
-                style['Nb errors_foreground-color'] = '#D5000D'
-            
-            rows.append((oid,
-                {
-                    'Site': site_name,
-                    'Nb errors': nb_err,
-                    'activate_oid': oid,
-                    '_style': style
-                }
-            ))
-        
-        return rows
-
-
-class ShowSiteSyncErrors(flow.Action):
-
-    error_counter = flow.Child(SyncErrorCounter).ui(
-        label='Errors',
-        expanded=True)
-
-    def get_buttons(self):
-        self.message.set('<h2>Synchronization errors</h2>')
-        return ['Refresh', 'Close']
-    
-    def run(self, button):
-        if button == 'Close':
-            return
-        
-        self.error_counter.refresh_error_count()
-        self.error_counter.touch()
-        return self.get_result(close=False)
-
-
 class ResetJobStatuses(flow.Action):
     _site = flow.Parent()
     status = flow.Param("WAITING", JobStatus)
@@ -1042,28 +964,28 @@ class ComputedBoolValue(flow.values.ComputedValue):
     DEFAULT_EDITOR = 'bool'
 
 
-class Site(flow.Object):
+class Site(CustomEntity):
 
-    short_name = flow.Param("")
-    description = flow.Param("")
-    # Define conditions required for a site to be considered as correctly configured
-    configured = flow.Computed().ui(editor='bool')
+    short_name    = Property()
+    code          = Property()
+    description   = Property()
+    configured    = flow.Computed().ui(editor='bool') # What a site needs to be considered configured
     configuration = flow.Child(ConfigureSite)
 
-    is_active = flow.BoolParam(True)
+    is_active     = Property().ui(editor='bool')
 
     def compute_child_value(self, child_value):
         if child_value is self.configured:
             self.configured.set(True)
+    
+    def get_code(self):
+        return self.code.get() or self.name()
 
 
-class SiteMap(flow.Map):
+class SiteMap(CustomEntityCollection):
 
     create_site = flow.Child(CreateSite)
 
-    _short_names = flow.HashParam()
-    _active_site_names = flow.OrderedStringSetParam()
-
     @classmethod
     def mapped_type(cls):
         return Site
@@ -1074,33 +996,6 @@ class SiteMap(flow.Map):
     def columns(self):
         return ["Site"]
     
-    def short_names(self):
-        short_names = []
-        
-        for name in self.mapped_names():
-            short_names.append(self.get_short_name(name))
-        
-        return short_names
-    
-    def get_short_name(self, site_name):
-        return self._short_names.get_key(site_name)
-    
-    def update_active_site_names(self):
-        self._active_site_names.revert_to_default()
-        i = 0
-
-        for s in self.mapped_items():
-            if s.is_active.get():
-                self._active_site_names.add(s.name(), i)
-            
-            i += 1
-    
-    def update_site_short_names(self):
-        self._short_names.revert_to_default()
-
-        for s in self.mapped_items():
-            self._short_names.set_key(s.name(), s.short_name.get())
-    
     def get_site_names(self, use_custom_order=False, active_only=False, short_names=False):
         sites_data = self.root().project().admin.multisites.sites_data.get()
         
@@ -1139,7 +1034,7 @@ class SiteMap(flow.Map):
 
 class ConfigureWorkingSite(ConfigureSite):
 
-    site_type = flow.Param("Studio", StaticSiteTypeChoices)
+    site_type = flow.SessionParam("Studio", SiteTypeSessionChoiceValue)
 
     root_windows_folder = flow.SessionParam("")
     root_linux_folder = flow.SessionParam("")
@@ -1194,7 +1089,7 @@ class GotoCurrentSiteQueue(flow.Action):
         return False
     
     def allow_context(self, context):
-        return context and context.endswith('.details')
+        return context and context.endswith('.inline')
     
     def run(self, button):
         current_site = self.root().project().get_current_site()
@@ -1356,39 +1251,40 @@ class SiteJobsPoolNames(PresetValue):
     
     def choices(self):
         site = self.root().project().get_current_site()
-        return ['default'] + site.pool_names.get()
+        return ['default'] + (site.pool_names.get() or [])
 
 
 class WorkingSite(Site):
     
     _site_map = flow.Parent()
     
-    site_type = flow.Param("Studio", StaticSiteTypeChoices)
-    request_files_from_anywhere = flow.BoolParam(False).ui(
+    site_type                    = Property(SiteTypeChoiceValue)
+    request_files_from_anywhere  = Property().ui(
+        editor='bool',
         tooltip=(
             "Allow the site to request files for any other site. "
             "Temporary option as long as synchronisation is manual."
         )
     )
-    is_kitsu_admin = flow.BoolParam(False)
-    auto_upload_kitsu_playblasts = flow.BoolParam(True)
 
-    root_folder = flow.Computed(cached=True)
-    root_windows_folder = flow.Param()
-    root_linux_folder = flow.Param()
-    root_darwin_folder = flow.Param()
+    is_kitsu_admin               = Property().ui(editor='bool')
+    auto_upload_kitsu_playblasts = Property().ui(editor='bool')
+
+    root_folder         = flow.Computed(cached=True)
+    root_windows_folder = Property()
+    root_linux_folder   = Property()
+    root_darwin_folder  = Property()
     
-    ordered_site_names = flow.Computed(cached=True)
-    custom_site_order = flow.Param('').watched().ui(
+    ordered_site_names  = flow.Computed(cached=True)
+    custom_site_order   = Property().watched().ui(
         label='Custom order',
         tooltip='Manage order in which sites are listed in the interface')
     
-    sync_dl_max_connections = flow.IntParam(1)
-    sync_up_max_connections = flow.IntParam(1)
-    pool_names = flow.OrderedStringSetParam()
+    sync_dl_max_connections = Property().ui(editor='int')
+    sync_up_max_connections = Property().ui(editor='int')
+    pool_names = Property()
 
     queue = flow.Child(Queue)
-
     configuration = flow.Child(ConfigureWorkingSite)
     goto_queue = flow.Child(GotoQueue).ui(
         label="Show job queue"
@@ -1397,13 +1293,15 @@ class WorkingSite(Site):
     site_environment = flow.Child(SiteEnvironment)
 
     with flow.group('AfterEffects settings'):
-        ae_render_settings_templates = flow.HashParam().ui(
+        ae_render_settings_templates = Property().ui(
+            editor='mapping',
             label='Render settings templates')
-        ae_output_module_templates   = flow.HashParam().ui(
+        ae_output_module_templates   = Property().ui(
+            editor='mapping',
             label='Output module templates')
-        ae_output_module_audio       = flow.Param().ui(
+        ae_output_module_audio       = Property().ui(
             label='Audio output module template')
-        ae_comp_name_patterns        = flow.OrderedStringSetParam().ui(
+        ae_comp_name_patterns        = Property().ui(
             label='Composition name patterns')
 
     def compute_child_value(self, child_value):
@@ -1564,11 +1462,11 @@ class ExchangeSite(Site):
 
     ICON = ('icons.libreflow', 'exchange')
 
-    server_url = flow.Param("")
-    server_login = flow.Param("")
-    server_password = flow.Param("").ui(editor='password')
-    bucket_name = flow.Param('')
-    enable_tls = flow.BoolParam(False)
+    server_url      = Property()
+    server_login    = Property()
+    server_password = Property().ui(editor='password')
+    bucket_name     = Property()
+    enable_tls      = Property().ui(editor='bool')
 
     sync_manager = flow.Child(SyncManager).ui(hidden=True)
 
@@ -1593,7 +1491,7 @@ class ExchangeSites(SiteMap):
 
     @classmethod
     def mapped_type(cls):
-        return ExchangeSite
+        return flow.injection.injectable(ExchangeSite)
 
     def mapped_names(self, page_num=0, page_size=None):
         return ["default_exchange"] + super(SiteMap, self).mapped_names(page_num, page_size)
@@ -1640,327 +1538,6 @@ class SyncMap(flow.DynamicMap):
         row['Name'] = name
 
 
-class RequestedRevisions(flow.DynamicMap):
-    
-    STYLE_BY_STATUS = {
-        'Available': ('#45cc3d', ('icons.libreflow', 'blank')),
-        'Requested': ('#b9c2c8', ('icons.libreflow', 'exclamation-sign-colored')),
-        'NotAvailable': ('#cc3b3c', ('icons.libreflow', 'blank')),
-        'Undefined': ('#cc3b3c', ('icons.libreflow', 'unavailable')),
-    }
-    
-    request_action = flow.Parent()
-    oids = flow.SessionParam([]).ui(editor='set')
-    
-    def mapped_names(self, page_num=0, page_size=None):
-        return self.oids.get()
-    
-    def get_mapped(self, name):
-        if not self.has_mapped_name(name):
-            raise MappedNameError(self.oid(), name)
-
-        try:
-            obj = self._mng.get_object(name)
-        except ValueError:
-            raise
-
-        return obj
-    
-    def columns(self):
-        return ['Revision']
-    
-    def _fill_row_cells(self, row, item):
-        row['Revision'] = item.oid()
-    
-    def _fill_row_style(self, style, item, row):
-        status = 'Undefined'
-        
-        if hasattr(item, 'request_as'):
-            status = item.get_sync_status(site_name=self.request_action.sites.target_site.get())
-            
-        style['Revision_foreground-color'] = self.STYLE_BY_STATUS[status][0]
-        style['Revision_icon'] = self.STYLE_BY_STATUS[status][1]
-
-
-class RequestRevisions(flow.Action):
-    
-    ICON = ('icons.gui', 'share-option')
-
-    pattern = flow.SessionParam("").ui(
-        placeholder="Revision oid pattern"
-    )
-    
-    sites = flow.Child(SiteSelection).ui(expanded=True)
-    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
-    
-    def allow_context(self, context):
-        current_site = self.root().project().get_current_site()
-        return context and current_site.request_files_from_anywhere.get()
-    
-    def _get_oid(self, pattern_str):
-        oids = []
-        patterns = []
-        
-        for pattern in pattern_str.split(';'):
-            patterns += self.resolve_pattern(pattern)
-        
-        for pattern in patterns:
-            oids += self.glob(self.root().project().oid(), pattern, 0)
-        
-        return oids
-
-    def get_buttons(self):
-        self.sites.source_site.set(self.root().project().get_current_site().name())
-        target_site_choices = self.sites.target_site.choices()
-        
-        if not target_site_choices:
-            msg = self.message.get()
-            if msg is None:
-                msg = ''
-            
-            msg += (
-                '<h3><font color="#D5000D">Making requests is not possible since '
-                'there is no other site defined for this project.</font></h3>'
-            )
-            self.message.set(msg)
-            
-            return ['Close']
-        
-        self.sites.target_site.set(target_site_choices[0])
-
-        return ["Request", "Refresh revision list", "Close"]
-    
-    def ls(self, root_oid):
-        related_info, mapped_names = self.root().session().cmds.Flow.ls(root_oid)
-        relation_oids = [rel_info[0] for rel_info in related_info]
-        mapped_oids = ["%s/%s" % (root_oid, name) for name in mapped_names]
-        
-        return relation_oids + mapped_oids
-    
-    def get_last_publication(self, file_oid):
-        o = self.root().get_object(file_oid)
-        
-        try:
-            head = o.get_head_revision()
-        except AttributeError:
-            return None
-        
-        if not head:
-            return None
-        
-        return head.name()
-    
-    def glob(self, root_oid, pattern, level):
-        if level >= pattern.count("/") - 1:
-            return [root_oid]
-
-        matches = []
-        level_pattern = "/".join(pattern.split("/")[:level + 3])
-
-        if level_pattern.endswith("[last]"):
-            file_oid = self.root().session().cmds.Flow.resolve_path(root_oid + "/../..")
-            head_name = self.get_last_publication(file_oid)
-
-            if not head_name:
-                return []
-            
-            pattern = pattern.replace("[last]", head_name)
-            level_pattern = level_pattern.replace("[last]", head_name)
-        
-        for oid in self.ls(root_oid):
-            if fnmatch.fnmatch(oid, level_pattern):
-                matches += self.glob(oid, pattern, level + 1)
-        
-        return matches
-    
-    def resolve_pattern(self, pattern_oid):
-        '''
-        Returns a list of patterns which correspond to all substitution combinations of `pattern_oid`.
-        Substitute string are specified in `pattern_oid` between brackets and separated with comas (`{s0, s1, ...}`).
-        '''
-        match = re.search(r'{[^{}]+}', pattern_oid)
-
-        if not match:
-            return [pattern_oid]
-
-        substitutes = match.group()[1:-1].split(',')
-        substitutes = [sub.replace(' ', '') for sub in substitutes]
-
-        sub_oids = [pattern_oid.replace(match.group(), sub, 1) for sub in substitutes]
-        oids = []
-
-        for sub_oid in sub_oids:
-            oids += self.resolve_pattern(sub_oid)
-        
-        return oids
-    
-    def run(self, button):
-        if button == "Close":
-            return
-        elif button == "Refresh revision list":
-            self.revisions.oids.revert_to_default()
-            oids = self._get_oid(self.pattern.get())
-            self.revisions.oids.set(oids)
-            self.revisions.touch()
-            
-            return self.get_result(close=False)
-        
-        source_site = self.sites.source_site.get()
-        target_site = self.sites.target_site.get()
-        
-        for obj in self.revisions.mapped_items():
-            # Skip objects which are not file revisions
-            if not hasattr(obj, 'request_as'):
-                continue
-            # Skip revisions which are not available on selected source site
-            if obj.get_sync_status(site_name=source_site) != 'Available':
-                continue
-
-            # Skip revisions which are already available on selected target site
-            status = obj.get_sync_status(site_name=target_site)
-            if status == 'Requested' or status == 'Available':
-                continue
-            
-            obj.request_as.sites.source_site.set(source_site)
-            obj.request_as.sites.target_site.set(target_site)
-            obj.request_as.include_dependencies.set(False)
-            obj.request_as.run(None)
-        
-        self.revisions.touch()
-
-        return self.get_result(close=False)
-
-
-class SiteMultiSelection(flow.Object):
-    
-    source_site = flow.Param('Auto select', ActiveSiteAutoSelectChoiceValue).watched()
-    target_sites = flow.Param([], ActiveSitesMultichoiceValue)
-    
-    def child_value_changed(self, child_value):
-        if child_value is self.source_site:
-            source_site = self.source_site.get()
-            
-            if source_site == 'Auto select':
-                self.target_sites.exclude_choice.set(None)
-            else:
-                self.target_sites.exclude_choice.set(self.source_site.get())
-            
-            self.target_sites.touch()
-
-
-class MultiRequestedRevisions(RequestedRevisions):
-    
-    STYLE_BY_STATUS = {
-        'Available': ('#45cc3d', ('icons.libreflow', 'checked-symbol-colored')),
-        'Requested': ('#b9c2c8', ('icons.libreflow', 'exclamation-sign-colored')),
-        'NotAvailable': ('#cc3b3c', ('icons.libreflow', 'blank')),
-        'Undefined': ('#cc3b3c', ('icons.libreflow', 'unavailable')),
-    }
-    
-    request_action = flow.Parent()
-    source_sites = flow.SessionParam([]).ui(hidden=True)
-    
-    def columns(self):
-        site_names = self.request_action.sites.target_sites.get() + self.source_sites.get()
-        site_names = list(dict.fromkeys(site_names))
-
-        return ['Revision'] + site_names
-    
-    def _fill_row_cells(self, row, item):
-        row['Revision'] = item.oid()
-        
-        for col in self.columns()[1:]:
-            row[col] = ''
-    
-    def _fill_row_style(self, style, item, row):
-        style['Revision_icon'] = ('icons.libreflow', 'blank')
-        
-        if not hasattr(item, 'request_as'):
-            style['Revision_icon'] = self.STYLE_BY_STATUS['Undefined'][1]
-            return
-        
-        site_names = self.request_action.sites.target_sites.get() + self.source_sites.get()
-        site_names = list(dict.fromkeys(site_names))
-
-        for site_name in site_names:
-            status = item.get_sync_status(site_name=site_name)
-            style[site_name + '_icon'] = self.STYLE_BY_STATUS[status][1]
-
-
-class MultiRequestRevisions(RequestRevisions):
-    
-    ICON = ('icons.libreflow', 'multi-share-option')
-    
-    sites = flow.Child(SiteMultiSelection).ui(expanded=True)
-    revisions = flow.Child(MultiRequestedRevisions).ui(expanded=True)
-    
-    def get_buttons(self):
-        self.sites.source_site.revert_to_default()
-        target_sites_choices = self.sites.target_sites.choices()
-        
-        if not target_sites_choices:
-            msg = self.message.get()
-            if msg is None:
-                msg = ''
-            
-            msg += (
-                '<h3><font color="#D5000D">Making requests is not possible since '
-                'there is no other site defined for this project.</font></h3>'
-            )
-            self.message.set(msg)
-            
-            return ['Close']
-
-        return ['Request', 'Refresh revision list', 'Close']
-    
-    def run(self, button):
-        if button == 'Close':
-            return
-        elif button == 'Refresh revision list':
-            self.revisions.oids.revert_to_default()
-            oids = self._get_oid(self.pattern.get())
-            self.revisions.oids.set(oids)
-
-            source_sites = []
-            for oid in oids:
-                site_name = self.root().session().cmds.Flow.get_value(oid + '/site')
-                source_sites.append(site_name)
-            
-            self.revisions.source_sites.set(source_sites)
-            self.revisions.touch()
-            
-            return self.get_result(close=False)
-        
-        source_site = self.sites.source_site.get()
-        auto_select_enabled = (source_site == 'Auto select')
-        
-        for obj in self.revisions.mapped_items():
-            # Skip objects which are not file revisions
-            if not hasattr(obj, 'request_as'):
-                continue
-            # Skip revisions which are not available on selected source site
-            if not auto_select_enabled and obj.get_sync_status(site_name=source_site) != 'Available':
-                continue
-            
-            for target_site in self.sites.target_sites.get():
-                # Skip revisions which are already available on selected target site
-                status = obj.get_sync_status(site_name=target_site)
-                if status == 'Requested' or status == 'Available':
-                    continue
-                
-                if not auto_select_enabled:
-                    obj.request_as.sites.source_site.set(source_site)
-                
-                obj.request_as.sites.target_site.set(target_site)
-                obj.request_as.sites.auto_select_enabled.set(auto_select_enabled)
-                obj.request_as.include_dependencies.set(False)
-                obj.request_as.run(None)
-        
-        self.revisions.touch()
-
-        return self.get_result(close=False)
-
-
 class UploadRevision(flow.Action):
 
     ICON = ('icons.libreflow', 'upload')
