@@ -8,6 +8,7 @@ from ..utils.kabaret.flow_entities.entities import EntityView
 from .file import FileSystemMap, TrackedFile, FileSystemRefs
 from .maputils import SimpleCreateAction
 from .task_manager import CreateTaskDefaultFiles, ManageTasksAction, FileTypeValue
+from .users import ToggleBookmarkAction
 
 
 class IconSize(enum.Enum):
@@ -26,6 +27,7 @@ class Task(Entity):
     """
     
     display_name = Property().ui(hidden=True)
+    code         = Property().ui(hidden=True)
     enabled      = Property().ui(hidden=True, editor='bool')
     icon_small   = Property().ui(hidden=True)
     icon_medium  = Property().ui(hidden=True)
@@ -54,7 +56,8 @@ class Task(Entity):
         if context_name == 'settings':
             return dict(
                 task=self.name(),
-                task_display_name=self.display_name.get(),
+                task_display_name=self.get_display_name(),
+                task_code=self.get_code()
             )
     
     def get_sized_icon(self, size=IconSize.SMALL):
@@ -68,10 +71,10 @@ class Task(Entity):
         return icon and tuple(icon)
     
     def get_display_name(self):
-        name = self.display_name.get()
-        if not name:
-            name = self.name()
-        return name
+        return self.display_name.get() or self.name()
+    
+    def get_code(self):
+        return self.code.get() or self.name()
     
     def get_primary_files(self):
         '''
@@ -199,6 +202,7 @@ class ManagedTask(Task):
     current_subtask        = Property().ui(hidden=True)
     color                  = Property().ui(hidden=True)
     
+    toggle_bookmark = flow.Child(ToggleBookmarkAction).ui(hidden=True)
     create_dft_files = flow.Child(CreateTaskDefaultFiles).ui(
         label='Create default files'
     )
@@ -220,6 +224,14 @@ class ManagedTask(Task):
             icon = tm.get_task_icon(self.name())
         
         return icon
+    
+    def get_code(self):
+        code = self.code.get()
+        if not code:
+            tm = self.root().project().get_task_manager()
+            code = tm.get_task_code(self.name())
+        
+        return code
 
     def is_assigned(self, user_name, subtask_name=None):
         assigned_users = self._get_assigned_users(subtask_name)
@@ -253,7 +265,7 @@ class ManagedTask(Task):
         color = self.color.get()
         if not color:
             tm = self.root().project().get_task_manager()
-            color = tm.get_task_color(self.name())
+            color = tm.get_task_color(self.name()) or None
         
         return color
     
