@@ -18,7 +18,21 @@ from libreflow.baseflow.runners import CHOICES_ICONS
 from ...resources.icons import gui as _
 
 
-class CheckableComboBox(QtWidgets.QComboBox):
+class LabelIcon(QtWidgets.QLabel):
+
+    def __init__(self, icon=None):
+        QtWidgets.QLabel.__init__(self, '')
+        if icon:
+            self.setIcon(icon)
+    
+    def setIcon(self, icon):
+        icon = QtGui.QIcon(resources.get_icon(icon))
+        pixmap = icon.pixmap(QtCore.QSize(16, 16))
+        self.setPixmap(pixmap)
+        self.setAlignment(QtCore.Qt.AlignVCenter)
+
+
+class FilterCheckableComboBox(QtWidgets.QComboBox):
 
     # Subclass Delegate to increase item height
     class Delegate(QtWidgets.QStyledItemDelegate):
@@ -49,7 +63,7 @@ class CheckableComboBox(QtWidgets.QComboBox):
         )
 
         # Use custom delegate
-        self.setItemDelegate(CheckableComboBox.Delegate())
+        self.setItemDelegate(FilterCheckableComboBox.Delegate())
 
         # Update the text when an item is toggled
         self.model().dataChanged.connect(self.updateText)
@@ -212,6 +226,215 @@ class CheckableComboBox(QtWidgets.QComboBox):
         return res
 
 
+class FilesUploadComboBox(QtWidgets.QComboBox):
+
+    # Subclass Delegate to increase item height
+    class Delegate(QtWidgets.QStyledItemDelegate):
+        def sizeHint(self, option, index):
+            size = super().sizeHint(option, index)
+            return size
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        # Make the combo editable to set a custom text, but readonly
+        self.setEditable(True)
+        self.lineEdit().setReadOnly(True)
+        # Make the lineedit the same color as QPushButton
+        self.setStyleSheet(
+            '''QComboBox {
+                background: #3e4041;
+            }'''
+        )
+
+        # Use custom delegate
+        self.setItemDelegate(FilesUploadComboBox.Delegate())
+
+        # Update the text when an item is toggled
+        self.model().dataChanged.connect(self.updateText)
+        # self.model().itemChanged.connect(self.handleItemChanged)
+
+        # Hide and show popup when clicking the line edit
+        self.lineEdit().installEventFilter(self)
+        self.closeOnLineEditClick = False
+
+        # Prevent popup from closing when clicking on an item
+        self.view().viewport().installEventFilter(self)
+
+        # Disable right-click
+        self.setContextMenuPolicy(QtCore.Qt.PreventContextMenu)
+
+        self.setAcceptDrops(True)
+
+        self.primary_files_names = []
+        self.previousData = []
+
+        self.addItem('- Add files', checkable=False)
+        self.addItem('- Clear selection', checkable=False)
+        self.setCurrentIndex(-1)
+
+        self.add_primary_files()
+
+        self.currentIndexChanged.connect(self._on_files_index_changed)
+
+    def resizeEvent(self, event):
+        # Recompute text to elide as needed
+        self.updateText()
+        super().resizeEvent(event)
+
+    def wheelEvent(self, *args, **kwargs):
+        pass
+
+    def eventFilter(self, object, event):
+        if object == self.lineEdit():
+            if event.type() == QtCore.QEvent.MouseButtonRelease:
+                if event.button() == QtCore.Qt.LeftButton:
+                    if self.closeOnLineEditClick:
+                        self.hidePopup()
+                    else:
+                        self.showPopup()
+                    return True
+        return False
+
+    def showPopup(self):
+        super().showPopup()
+        # When the popup is displayed, a click on the lineedit should close it
+        self.closeOnLineEditClick = True
+
+    def hidePopup(self):
+        super().hidePopup()
+        # Used to prevent immediate reopening when clicking on the lineEdit
+        self.startTimer(100)
+        # Refresh the display text when closing
+        self.updateText()
+        # Check if there are any changes
+        newRes = self.currentData()
+        if self.previousData != newRes:
+            self.previousData = newRes
+
+    def timerEvent(self, event):
+        # After timeout, kill timer, and reenable click on line edit
+        self.killTimer(event.timerId())
+        self.closeOnLineEditClick = False
+
+    def dragEnterEvent(self, event):
+        if event.mimeData().hasUrls:
+            event.accept()
+        else:
+            event.ignore()
+
+    def dragMoveEvent(self, event):
+        if event.mimeData().hasUrls:
+            event.setDropAction(QtCore.Qt.CopyAction)
+            event.accept()
+        else:
+            event.ignore()
+
+    def dropEvent(self, event):
+        if event.mimeData().hasUrls:
+            links = []
+            for url in event.mimeData().urls():
+                links.append(str(url.toLocalFile()))
+            self.add_files(links)
+        else:
+            event.ignore()
+
+    def updateText(self):
+        texts = []
+        for i in range(self.model().rowCount()):
+            if self.model().item(i).checkState() == QtCore.Qt.Checked:
+                texts.append(self.model().item(i).text())
+        text = ", ".join(texts)
+
+        # Compute elided text (with "...")
+        metrics = QtGui.QFontMetrics(self.lineEdit().font())
+        elidedText = metrics.elidedText(text, QtCore.Qt.ElideRight, self.lineEdit().width())
+        if '…' in elidedText:
+            elidedText = str(len(texts)) + ' Files'
+        self.lineEdit().setText(elidedText)
+
+    def addItem(self, text, data=None, checkable=True):
+        item = QtGui.QStandardItem()
+        item.setText(text)
+        if data is None:
+            item.setData(text)
+        else:
+            item.setData(data)
+        if checkable:
+            item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsUserCheckable)
+            item.setData(QtCore.Qt.Unchecked, QtCore.Qt.CheckStateRole)
+        self.model().appendRow(item)
+
+    def addItems(self, texts, datalist=None):
+        for i, text in enumerate(texts):
+            try:
+                data = datalist[i]
+            except (TypeError, IndexError):
+                data = None
+            self.addItem(text, data)
+
+    def setChecked(self, texts, state):
+        for i, text in enumerate(texts):
+            for row in range(self.model().rowCount()):
+                if self.model().item(row).text() == text:
+                    if state == True:
+                        self.model().item(row).setCheckState(QtCore.Qt.Checked)
+                    else:
+                        self.model().item(row).setCheckState(QtCore.Qt.Unchecked)
+
+    def currentData(self):
+        # Return the list of selected items data
+        res = []
+        for i in range(self.model().rowCount()):
+            if self.model().item(i).checkState() == QtCore.Qt.Checked:
+                res.append(self.model().item(i).data())
+        return res
+
+    def add_primary_files(self):
+        files_list = self.parent().task_item.files_list
+        for i in range(files_list.topLevelItemCount()):
+            f = files_list.topLevelItem(i)
+            check = self.parent().page_widget.is_uploadable(f.file_name)
+            if check:
+                r = self.parent().page_widget.session.cmds.Flow.call(
+                    f.file_oid, 'get_head_revision', ['Available'], {}
+                )
+                self.addItem(f.file_name, r.get_path())
+                self.primary_files_names.append(f.file_name)
+
+    def add_files(self, paths):
+        for path in paths:
+            exist = False
+            for i in range(self.model().rowCount()):
+                if self.model().item(i).data() == path:
+                    self.model().item(i).setCheckState(QtCore.Qt.Checked)
+                    exist = True
+                    break
+            if not exist:
+                filename = os.path.split(path)[1]
+                self.addItem(filename, path)
+                index = self.findText(filename, QtCore.Qt.MatchFixedString)
+                self.model().item(index).setCheckState(QtCore.Qt.Checked)
+
+    def _on_files_index_changed(self, index):
+        if index == 0:
+            dialog = QtWidgets.QFileDialog()
+            dialog.setFileMode(QtWidgets.QFileDialog.ExistingFiles)
+
+            if dialog.exec_():
+                paths = dialog.selectedFiles()
+                self.add_files(paths)
+            else:
+                self.setCurrentIndex(-1)
+                self.updateText()
+
+        if index == 1:
+            self.setCurrentIndex(-1)
+            for i in range(self.model().rowCount()):
+                if self.model().item(i).checkState() == QtCore.Qt.Checked:
+                    self.model().item(i).setCheckState(QtCore.Qt.Unchecked)
+
+
 class NavigationButton(QtWidgets.QToolButton):
 
     def __init__(self, name, oid):
@@ -221,119 +444,20 @@ class NavigationButton(QtWidgets.QToolButton):
         else:
             self.name = name
         self.oid = oid
-        # self.nav_widget = nav_widget
-        # self._last_click_pos = QtCore.QPoint()
 
-        # self.setFont(self.nav_widget.bt_font)
         self.setProperty('tight_layout', True)
         self.setProperty('hide_arrow', True)
         self.setProperty('no_border', True)
         self.setProperty('square', True)
         self.setArrowType(QtCore.Qt.NoArrow)
-        # self.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
 
         self.setText('/%s' % (self.name,))
 
         self.clicked.connect(self._goto)
 
-        # self._menu = PopupMenu(self)
-
-    # def _goto_oid(self, oid):
-    #     self.nav_widget._goto(oid)
-
     def _goto(self, b=None):
         self.parent().tasks_list.page_widget.page.goto(self.oid)
 
-    # def _show_navigables_menu(self, full_oid):
-    #     self._menu.clear()
-
-    #     m = self._menu
-
-    #     m.addAction('Loading...')
-    #     # self.setMenu(m)
-    #     # self.showMenu()
-
-    #     session = self.nav_widget._navigator.session
-    #     try:
-    #         navigatable_entries = session.cmds.Flow.get_navigable_oids(
-    #             self.oid, full_oid
-    #         )
-    #     except Exception as err:
-    #         m.clear()
-    #         m.addAction('ERROR: ' + str(err))
-    #         raise
-
-    #     m.clear()
-    #     root = m
-    #     for i, entry in enumerate(navigatable_entries, 1):
-    #         if entry is None:
-    #             m.addMenu()
-    #         else:
-    #             label, oid = entry
-    #             if oid == self.oid:
-    #                 item = m.addItem(
-    #                     '> %s <' % label
-    #                 )
-    #                 font = item.font()
-    #                 font.setBold(True)
-    #                 item.setFont(font)
-    #             else:
-    #                 m.addAction(label, callback=functools.partial(self._goto_oid, oid))
-
-    #     root.addMenu()
-    #     root.addAction("Copy", icon=resources.get_icon(('icons.gui', 'copy-document')),
-    #                    callback=lambda: QtWidgets.QApplication.clipboard().setText(self.oid))
-    #     root.addAction("Paste", icon=resources.get_icon(('icons.gui', 'paste-from-clipboard')),
-    #                    callback=lambda: self._goto_oid(QtWidgets.QApplication.clipboard().text()))
-    #     m.popup(QtGui.QCursor.pos())
-
-    # def mousePressEvent(self, e):
-    #     if e.button() == QtCore.Qt.RightButton:
-    #         self._show_navigables_menu(self.nav_widget.current_oid())
-    #     else:
-    #         self._last_click_pos = e.pos()
-    #     super(NavigationButton, self).mousePressEvent(e)
-
-    # def mouseMoveEvent(self, e):
-    #     super(NavigationButton, self).mouseMoveEvent(e)
-
-    #     if self._last_click_pos.isNull():
-    #         return
-    #     drag_distance = (e.pos() - self._last_click_pos).manhattanLength()
-    #     if drag_distance < QtWidgets.QApplication.startDragDistance():
-    #         return
-
-    #     oids = [self.oid]
-    #     mime_data = QtCore.QMimeData()
-    #     md = self.nav_widget._navigator.session.cmds.Flow.to_mime_data(oids)
-    #     for data_type, data in six.iteritems(md):
-    #         mime_data.setData(data_type, data)
-
-    #     pixmap = QtGui.QPixmap(self.size())
-    #     self.render(pixmap)
-
-    #     # below makes the pixmap half transparent
-    #     painter = QtGui.QPainter(pixmap)
-    #     painter.setCompositionMode(painter.CompositionMode_DestinationIn)
-    #     painter.fillRect(pixmap.rect(), QtGui.QColor(0, 0, 0, 127))
-    #     painter.end()
-
-    #     # make a QDrag
-    #     drag = QtGui.QDrag(self)
-    #     drag.setMimeData(mime_data)
-    #     drag.setPixmap(pixmap)
-
-    #     # shift the Pixmap so that it coincides with the cursor position
-    #     drag.setHotSpot(self._last_click_pos)
-
-    #     # start the drag operation
-    #     # exec_ will return the accepted action from dropEvent
-    #     drag_result = drag.exec_(QtCore.Qt.CopyAction)
-
-    # def mouseReleaseEvent(self, e):
-    #     self._last_click_pos = QtCore.QPoint()
-    #     super(NavigationButton, self).mouseReleaseEvent(e)
-
 
 # class KitsuUser(QtWidgets.QWidget):
 
@@ -384,7 +508,7 @@ class MyTasksFooter(QtWidgets.QWidget):
         self.left_text = QtWidgets.QLabel()
         self.left_text.setText(str(self.page_widget.list.get_count())+' Tasks')
         self.page_widget.list.get_count()
-        self.shift_label = QtWidgets.QLabel('Press SHIFT to display shot infos')
+        self.shift_label = QtWidgets.QLabel('Press SHIFT to display entity description')
 
         hlo = QtWidgets.QHBoxLayout()
         hlo.addWidget(self.left_text)
@@ -395,6 +519,93 @@ class MyTasksFooter(QtWidgets.QWidget):
 
     def refresh_count(self):
         self.left_text.setText(str(self.page_widget.list.get_count())+' Tasks')
+        self.left_text.setStyleSheet('')
+
+
+class EditStatusDialog(QtWidgets.QDialog):
+
+    def __init__(self, task_item):
+        super(EditStatusDialog, self).__init__(task_item)
+        self.task_item = task_item
+        self.page_widget = task_item.page_widget
+        self.setFocusPolicy(QtCore.Qt.StrongFocus)
+        self.setStyleSheet('font-size: 11px;')
+        self.setMaximumSize(600, 300)
+
+        self.build()
+    
+    def build(self):
+        self.layout = QtWidgets.QVBoxLayout(self)
+        self.layout.setContentsMargins(20,20,20,20)
+
+        palette = self.palette()
+        palette.setColor(QtGui.QPalette.Base, palette.color(QtGui.QPalette.Window))
+        self.setPalette(palette)
+
+        self.content_layout = QtWidgets.QGridLayout()
+        self.content_layout.setAlignment(QtCore.Qt.AlignTop)
+      
+        self.content_layout.addWidget(LabelIcon(('icons.flow', 'input')), 0, 0, QtCore.Qt.AlignVCenter)
+        self.content_layout.addWidget(QtWidgets.QLabel('Target Status'), 0, 1, QtCore.Qt.AlignVCenter)
+        self.target_status = QtWidgets.QComboBox()
+        self.target_status.addItems(sorted(self.page_widget.get_task_statutes(False)))
+        self.target_status.setCurrentText('Work In Progress')
+        self.content_layout.addWidget(self.target_status, 0, 2, QtCore.Qt.AlignVCenter)
+
+        self.content_layout.addWidget(LabelIcon(('icons.flow', 'input')), 1, 0, QtCore.Qt.AlignVCenter)
+        self.content_layout.addWidget(QtWidgets.QLabel('Comment'), 1, 1, QtCore.Qt.AlignVCenter)
+        self.comment = QtWidgets.QTextEdit('')
+        self.content_layout.addWidget(self.comment, 1, 2, QtCore.Qt.AlignVCenter)
+
+        self.content_layout.addWidget(LabelIcon(('icons.flow', 'input')), 2, 0, QtCore.Qt.AlignVCenter)
+        self.content_layout.addWidget(QtWidgets.QLabel('Files'), 2, 1, QtCore.Qt.AlignVCenter)
+        self.files = FilesUploadComboBox(self)
+        self.content_layout.addWidget(self.files, 2, 2, QtCore.Qt.AlignVCenter)
+
+        # Buttons
+        self.button_layout = QtWidgets.QHBoxLayout()
+
+        self.button_post = QtWidgets.QPushButton('Post')
+        self.button_cancel = QtWidgets.QPushButton('Cancel')
+
+        self.button_post.clicked.connect(self._on_post_button_clicked)
+        self.button_cancel.clicked.connect(self._on_cancel_button_clicked)
+
+        self.button_post.setAutoDefault(False)
+        self.button_cancel.setAutoDefault(False)
+
+        self.button_layout.addStretch()
+        self.button_layout.addWidget(self.button_post)
+        self.button_layout.addWidget(self.button_cancel)
+
+        self.layout.addLayout(self.content_layout)
+        self.layout.addLayout(self.button_layout)
+        self.setLayout(self.layout)
+
+    def sizeHint(self):
+        return QtCore.QSize(475, 275)
+
+    def _on_post_button_clicked(self):
+        if len(self.files.currentData()) == 1:
+            if self.files.currentText() in self.files.primary_files_names:
+                self.page_widget.upload_preview(
+                    self.page_widget.session.cmds.Flow.get_value(self.task_item.oid+'/entity_id'),
+                    self.task_item.task_type,
+                    self.target_status.currentText(),
+                    self.files.currentData()[0],
+                    self.comment.toPlainText()
+                )
+        else:
+            self.page_widget.set_task_status(
+                self.task_item.task_id,
+                self.target_status.currentText(),
+                self.comment.toPlainText(),
+                self.files.currentData()
+            )
+        self.page_widget.list.refresh(True)
+
+    def _on_cancel_button_clicked(self):
+        self.close()
 
 
 class TaskFile(QtWidgets.QTreeWidgetItem):
@@ -408,14 +619,14 @@ class TaskFile(QtWidgets.QTreeWidgetItem):
         self.refresh()
     
     def refresh(self):
-        file_name = self.page_widget.session.cmds.Flow.get_value(self.file_oid+'/display_name')
-        name, ext = os.path.splitext(file_name)
+        self.file_name = self.page_widget.session.cmds.Flow.get_value(self.file_oid+'/display_name')
+        name, ext = os.path.splitext(self.file_name)
         icon = CHOICES_ICONS.get(
             ext[1:], ('icons.gui', 'text-file-1')
         ) 
 
         self.setIcon(0, self.get_icon(icon))
-        self.setText(0, file_name)
+        self.setText(0, self.file_name)
 
         self.setExpanded(True)
     
@@ -463,9 +674,9 @@ class TaskFiles(QtWidgets.QTreeWidget):
         self.blockSignals(True)
         self.clear()
 
-        primary_files = self.page_widget.session.cmds.Flow.get_value(
+        primary_files = sorted(self.page_widget.session.cmds.Flow.get_value(
             self.task_item.oid+'/primary_files'
-        )
+        ))
 
         for oid in primary_files:
             TaskFile(self, oid)
@@ -504,9 +715,13 @@ class TaskItem(QtWidgets.QWidget):
 
         self.task_oid = self.page_widget.session.cmds.Flow.get_value(self.oid+'/task_oid')
         
-        self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
-        self.setStyleSheet('''QWidget#TaskItem {background-color: #3e4041;} QWidget {font-size: 12px;}''')
+        self.op = QtWidgets.QGraphicsOpacityEffect(self)
+        self.op.setOpacity(1.00)
+        self.setGraphicsEffect(self.op)
+        self.setAutoFillBackground(True)
+        self.setStyleSheet('font-size: 12px;')
         self.setMinimumHeight(165)
+        self.setMaximumHeight(200)
 
         self.build()
         self.refresh()
@@ -532,15 +747,17 @@ class TaskItem(QtWidgets.QWidget):
         kitsu_header = QtWidgets.QHBoxLayout()
         kitsu_header.setSpacing(0)
 
+        self.type_label = QtWidgets.QLabel('')
         self.status_label = QtWidgets.QLabel('')
-        self.status_label.setStyleSheet('padding-right: 1;')
         self.redirect_task = QtWidgets.QPushButton(QtGui.QIcon(resources.get_icon(('icons.libreflow', 'kitsu'))), '')
         self.redirect_task.setIconSize(QtCore.QSize(13,13))
         self.redirect_task.setStyleSheet('padding: 2;')
         self.edit_status = QtWidgets.QPushButton(QtGui.QIcon(resources.get_icon(('icons.libreflow', 'edit-blank'))), '')
         self.edit_status.setIconSize(QtCore.QSize(13,13))
         self.edit_status.setStyleSheet('padding: 2;')
+        self.edit_status.clicked.connect(self._on_edit_status_button_clicked)
         kitsu_header.addStretch()
+        kitsu_header.addWidget(self.type_label)
         kitsu_header.addWidget(self.status_label)
         kitsu_header.addWidget(self.redirect_task)
         kitsu_header.addWidget(self.edit_status)
@@ -550,15 +767,27 @@ class TaskItem(QtWidgets.QWidget):
         self.kitsu_comments = QtWidgets.QTextBrowser()
         self.kitsu_comments.setOpenExternalLinks(True)
         self.kitsu_comments.setReadOnly(True)
+        self.kitsu_comments.setPlaceholderText('No comment for this task.')
 
         self.kitsu_lo.addWidget(self.kitsu_comments)
         self.right_kitsu.setLayout(self.kitsu_lo)
 
-        ## Shot Data
-        self.right_shot = QtWidgets.QWidget()
-        shot_lo = QtWidgets.QGridLayout()
-        shot_lo.setContentsMargins(0,0,0,0)
-        shot_header = QtWidgets.QHBoxLayout()
+        ## Entity Data
+        self.right_entity = QtWidgets.QWidget()
+        entity_lo = QtWidgets.QGridLayout()
+        entity_lo.setContentsMargins(0,0,0,0)
+
+        self.header_spacer = QtWidgets.QWidget()
+        header_spacer_lo = QtWidgets.QVBoxLayout()
+        header_spacer_lo.setContentsMargins(0,0,0,0)
+        self.header_spacer.setLayout(header_spacer_lo)
+        self.spacer = QtWidgets.QSpacerItem(0, 23, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
+        header_spacer_lo.addItem(self.spacer)
+
+        self.shot_header = QtWidgets.QWidget()
+        shot_header_lo = QtWidgets.QHBoxLayout()
+        shot_header_lo.setContentsMargins(0,0,0,0)
+        self.shot_header.setLayout(shot_header_lo)
 
         self.shot_icon = QtWidgets.QLabel()
         pm = resources.get_pixmap('icons.gui', 'film-strip-with-two-photograms')
@@ -567,29 +796,30 @@ class TaskItem(QtWidgets.QWidget):
         self.shot_frames = QtWidgets.QLabel('Frames duration')
         self.shot_frames.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
 
-        shot_header.addWidget(self.shot_icon)
-        shot_header.addWidget(self.shot_frames)
-        shot_header.addStretch()
-        shot_lo.addLayout(shot_header, 0, 0, QtCore.Qt.AlignTop)
+        shot_header_lo.addWidget(self.shot_icon)
+        shot_header_lo.addWidget(self.shot_frames)
+        shot_header_lo.addStretch()
+        entity_lo.addWidget(self.shot_header, 0, 0, QtCore.Qt.AlignTop)
+        entity_lo.addWidget(self.header_spacer, 0, 0, QtCore.Qt.AlignTop)
 
-        self.shot_description = QtWidgets.QTextEdit('')
-        self.shot_description.setPlaceholderText('No description')
-        self.shot_description.setReadOnly(True)
+        self.entity_description = QtWidgets.QTextEdit('')
+        self.entity_description.setPlaceholderText('No description')
+        self.entity_description.setReadOnly(True)
         # Set line spacing
         descBlockFmt = QtGui.QTextBlockFormat()
         descBlockFmt.setLineHeight(120, QtGui.QTextBlockFormat.ProportionalHeight)
-        descTextCursor = self.shot_description.textCursor()
+        descTextCursor = self.entity_description.textCursor()
         descTextCursor.clearSelection()
         descTextCursor.select(QtGui.QTextCursor.Document)
         descTextCursor.mergeBlockFormat(descBlockFmt)
 
-        shot_lo.addWidget(self.shot_description, 2, 0, QtCore.Qt.AlignTop)
-        self.right_shot.setLayout(shot_lo)
-        self.right_shot.hide()
+        entity_lo.addWidget(self.entity_description, 2, 0, QtCore.Qt.AlignTop)
+        self.right_entity.setLayout(entity_lo)
+        self.right_entity.hide()
 
         container.addLayout(left_lo, 0, 0)
         container.addWidget(self.right_kitsu, 0, 1)
-        container.addWidget(self.right_shot, 0, 1)
+        container.addWidget(self.right_entity, 0, 1)
         container.setColumnStretch(0, 3)
         container.setColumnStretch(1, 2)
         self.setLayout(container)
@@ -598,21 +828,29 @@ class TaskItem(QtWidgets.QWidget):
         self.task_id = self.page_widget.session.cmds.Flow.get_value(self.oid+'/task_id')
         entity_type = self.page_widget.session.cmds.Flow.get_value(self.oid+'/entity_type')
 
+        # Bookmark
+        if self.page_widget.session.cmds.Flow.get_value(self.oid+'/is_bookmarked'):
+            self.bookmark_button = QtWidgets.QToolButton()
+            self.bookmark_button.setIcon(resources.get_icon(('icons.gui', 'star')))
+            self.bookmark_button.setIconSize(QtCore.QSize(14, 14))
+            self.bookmark_button.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
+            self.bookmark_button.clicked.connect(self._on_bookmark_button_clicked)
+            self.oid_lo.addWidget(self.bookmark_button)
+
         # Navigation buttons
         project_oid = self.page_widget.get_project_oid()
 
-        label_to_oid = self.page_widget.session.cmds.Flow.split_oid(
-            self.task_oid, True, project_oid
-        )
+        label_to_oid = self.page_widget.session.cmds.Flow.split_oid(self.task_oid, True, project_oid)
         for label, goto_oid in label_to_oid:
             nav_button = NavigationButton(label, goto_oid)
             self.oid_lo.addWidget(nav_button)
         self.oid_lo.addStretch()
 
         # Task Status
-        _type = self.page_widget.session.cmds.Flow.get_value(self.oid+'/task_type')
+        self.task_type = self.page_widget.session.cmds.Flow.get_value(self.oid+'/task_type')
         status = self.page_widget.session.cmds.Flow.get_value(self.oid+'/task_status')
-        self.status_label.setText(f'{_type} - {status}')
+        self.type_label.setText(f'{self.task_type} -')
+        self.status_label.setText(f'{status.upper()}')
 
         # Redirect Task
         task_link = self.create_task_link(entity_type)
@@ -621,46 +859,82 @@ class TaskItem(QtWidgets.QWidget):
         # Comments
         comments = self.page_widget.get_task_comments(self.task_id)
 
-        comment_html = '''
-        <style>
-            a:link {
-                color: #00BFFF;
-                background-color: transparent;
-                text-decoration: none;
-            }
-        </style>
-        '''
+        if comments:
+            comment_html = '''
+            <style>
+                a:link {
+                    color: #00BFFF;
+                    background-color: transparent;
+                    text-decoration: none;
+                }
+                .separator {
+                    border-bottom: 1px solid white;
+                    border-collapse: collapse;
+                }
+                .spacer {
+                    margin-bottom: 10px;
+                }
+            </style>
+            '''
 
-        for c in comments:
-            date_object = datetime.datetime.strptime(c['created_at'], "%Y-%m-%dT%H:%M:%S")
+            for i, c in enumerate(comments):
+                date_object = datetime.datetime.strptime(c['created_at'], "%Y-%m-%dT%H:%M:%S")
 
-            comment_html = comment_html + '''
-            <table cellspacing=0 width=100%>
-            <tr style="border:1px solid red;">
-                <td><span style='color: {color}; font-weight: bold;'>{status}</span> - {name}</td>
-                <td align=right>{date}</td>
-            </tr>
-            </table>
-            '''.format(
-                color=c['task_status']['color'],
-                status=c['task_status']['short_name'].upper(),
-                name=c['person']['first_name'] + ' ' + c['person']['last_name'],
-                date=date_object.strftime('%d/%m'),
-            )
+                comment_html = comment_html + '''
+                <table cellspacing=0 width=100%>
+                <tr>
+                    <td><span style='color: {color}; font-weight: bold;'>{status}</span> - {name}</td>
+                    <td align=right>{date}</td>
+                </tr>
+                </table>
+                '''.format(
+                    color=c['task_status']['color'],
+                    status=c['task_status']['short_name'].upper(),
+                    name=c['person']['first_name'] + ' ' + c['person']['last_name'],
+                    date=date_object.strftime('%d/%m'),
+                )
 
-            if c['text'] != '':
-                comment_html = comment_html + '''<p>{text}</p>'''.format(text=c['text'])
+                if c['text'] != '':
+                    if '\n' in c['text']:
+                        comment_lines = c['text'].split('\n')
+                        for line in comment_lines:
+                            comment_html = comment_html + '''<p>{text}</p>'''.format(text=line)
+                    else:
+                        comment_html = comment_html + '''<p>{text}</p>'''.format(text=c['text'])
 
-            if c['previews'] != []:
-                revision_link = self.create_revision_link(entity_type, c['previews'][0]['id'])
-                comment_html = comment_html + '''<p><a href='{link}'>Revision</a></p>'''.format(link=revision_link)
-            
-            comment_html = comment_html + '''<hr>'''
+                if c['previews'] != []:
+                    revision_link = self.create_revision_link(entity_type, c['previews'][0]['id'])
+                    comment_html = comment_html + '''<p><a href='{link}'>Revision</a></p>'''.format(link=revision_link)
+                
+                if i == 0:
+                    self.status_label.setStyleSheet(f'color: {c["task_status"]["color"]}; font-weight: 700; padding-right: 1;')
 
-        self.kitsu_comments.setHtml(comment_html)
+                if i == len(comments)-1:
+                    continue
+                comment_html = comment_html + '''<table cellspacing=0 class="spacer" width=100%><tr><td class="separator"/></tr></table>'''
 
-        self.shot_frames.setText(str(self.page_widget.session.cmds.Flow.get_value(self.oid+'/shot_frames')))
-        self.shot_description.setText(self.page_widget.session.cmds.Flow.get_value(self.oid+'/entity_description'))
+            self.kitsu_comments.setHtml(comment_html)
+
+        if self.status_label.styleSheet() == '':
+            status_data = self.page_widget.get_task_status(status)
+            self.status_label.setStyleSheet(f'color: {status_data["color"]}; font-weight: 700; padding-right: 1;')
+
+        if self.page_widget.session.cmds.Flow.get_value(self.oid+'/shot_frames'):
+            self.header_spacer.hide()
+            frames = self.page_widget.session.cmds.Flow.get_value(self.oid+'/shot_frames')
+            project_fps = int(self.page_widget.get_project_fps())
+            timecode = self.frames_to_timecode(frames, project_fps)
+            self.shot_frames.setText(str(frames)+' frames ('+timecode+')')
+        else:
+            self.shot_header.hide()
+        self.entity_description.setText(self.page_widget.session.cmds.Flow.get_value(self.oid+'/entity_description'))
+
+    def frames_to_timecode(self, frames, fps):
+        h = int(frames / 86400) 
+        m = int(frames / 1440) % 60
+        s = int((frames % 1440)/fps)
+        f = frames % 1440 % fps
+        return ( "%02d:%02d:%02d:%02d" % ( h, m, s, f))
 
     def create_task_link(self, entity_type):
         return '{server}/productions/{project}/{entity}/tasks/{task}'.format(
@@ -679,15 +953,21 @@ class TaskItem(QtWidgets.QWidget):
             preview=preview_id
         )
     
+    def _on_bookmark_button_clicked(self):
+        is_bookmarked = self.page_widget.toggle_bookmark(self.task_oid)
+        if is_bookmarked:
+            self.bookmark_button.setIcon(resources.get_icon(('icons.gui', 'star')))
+            self.bookmark_button.setIconSize(QtCore.QSize(12, 12))
+        else:
+            self.bookmark_button.setIcon(resources.get_icon(('icons.gui', 'star-1')))
+            self.bookmark_button.setIconSize(QtCore.QSize(12, 12))
+
     def _on_redirect_task_button_clicked(self, link):
         webbrowser.open(link)
-
-    # def addCommentMode(self):
-    #     self.kitsu_comment.setText('')
-    #     self.kitsu_comment_publish = QtWidgets.QPushButton(QtGui.QIcon(resources.get_icon(('icons.gui', 'share-option'))), '')
-    #     self.kitsu_comment_publish.setIconSize(QtCore.QSize(12,12))
-
-    #     self.kitsu_lo.addWidget(self.kitsu_comment_publish, 2, 0, QtCore.Qt.AlignRight | QtCore.Qt.AlignBottom)
+    
+    def _on_edit_status_button_clicked(self):
+        dialog = EditStatusDialog(self)
+        dialog.exec()
 
 
 class MyTasksList(QtWidgets.QScrollArea):
@@ -703,6 +983,12 @@ class MyTasksList(QtWidgets.QScrollArea):
     def refresh(self, force_update=False):
         if force_update:
             self.page_widget.footer.left_text.setText('Updating...')
+            self.page_widget.footer.left_text.setStyleSheet('color: red; font-weight: 700;')
+            for i in reversed(range(self.layout.count())):
+                widget = self.layout.itemAt(i).widget()
+                if widget is not None:
+                    widget.op.setOpacity(0.50)
+
             QtWidgets.QApplication.processEvents()
             QtWidgets.QApplication.processEvents()
 
@@ -731,15 +1017,15 @@ class MyTasksList(QtWidgets.QScrollArea):
     def get_count(self):
         return self.layout.count() - 1
 
-    def toggle_shot_data(self):
+    def toggle_entity_data(self):
         for i in reversed(range(self.layout.count())):
             task = self.layout.itemAt(i).widget()
             if task is not None:
                 if task.right_kitsu.isVisible():
                     task.right_kitsu.hide()
-                    task.right_shot.show()
+                    task.right_entity.show()
                 else:
-                    task.right_shot.hide()
+                    task.right_entity.hide()
                     task.right_kitsu.show()
 
 
@@ -758,9 +1044,9 @@ class MyTasksHeader(QtWidgets.QWidget):
         self.refresh_button.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
         
         self.filter_label = QtWidgets.QLabel('Filter by')
-        self.filter_combobox = CheckableComboBox()
+        self.filter_combobox = FilterCheckableComboBox()
         self.filter_combobox.addItem('Default')
-        self.filter_combobox.addItems(self.page_widget.get_task_statutes())
+        self.filter_combobox.addItems(sorted([task.upper() for task in self.page_widget.get_task_statutes(True)]))
         self.filter_combobox.setDefaultPreset()
         filter_value = self.page_widget.get_user_filter()
         if filter_value == []:
@@ -850,7 +1136,7 @@ class MyTasksPageWidget(CustomPageWidget):
         super(MyTasksPageWidget, self).keyPressEvent(event)
 
         if event.key() == QtCore.Qt.Key_Shift:
-            self.list.toggle_shot_data()
+            self.list.toggle_entity_data()
             self.key_press_start_time =  time.time()
 
     def keyReleaseEvent(self, event):
@@ -858,7 +1144,7 @@ class MyTasksPageWidget(CustomPageWidget):
         key_press_time = time.time() - self.key_press_start_time
 
         if event.key() == QtCore.Qt.Key_Shift and key_press_time > 0.5:
-            self.list.toggle_shot_data()
+            self.list.toggle_entity_data()
 
     def get_project_oid(self):
         return self.session.cmds.Flow.call(
@@ -870,6 +1156,11 @@ class MyTasksPageWidget(CustomPageWidget):
             self.oid, 'get_project_id', {}, {}
         )
 
+    def get_project_fps(self):
+        return self.session.cmds.Flow.call(
+            self.oid, 'get_project_fps', {}, {}
+        )
+
     def get_user_filter(self):
         return self.session.cmds.Flow.get_value(self.oid+'/settings/task_statues_filter')
 
@@ -886,11 +1177,35 @@ class MyTasksPageWidget(CustomPageWidget):
             self.oid, 'get_server_url', {}, {}
         )
 
-    def get_task_statutes(self):
-        statues = self.session.cmds.Flow.call(
-            self.oid, 'get_task_statutes', {}, {}
+    def is_uploadable(self, file_name):
+        return self.session.cmds.Flow.call(
+            self.oid, 'is_uploadable', [file_name], {}
+        )
+
+    def get_task_statutes(self, short_name):
+        return self.session.cmds.Flow.call(
+            self.oid, 'get_task_statutes', [short_name], {}
+        )
+
+    def get_task_status(self, task_status_name):
+        return self.session.cmds.Flow.call(
+            self.oid, 'get_task_status', [task_status_name], {}
+        )
+
+    def set_task_status(self, task_id, task_status_name, comment, files):
+        return self.session.cmds.Flow.call(
+            self.oid, 'set_task_status', [task_id, task_status_name, comment, files], {}
+        )
+
+    def upload_preview(self, entity_id, task_name, task_status_name, file_path, comment):
+        return self.session.cmds.Flow.call(
+            self.oid, 'upload_preview', [entity_id, task_name, task_status_name, file_path, comment], {}
+        )
+
+    def toggle_bookmark(self, oid):
+        return self.session.cmds.Flow.call(
+            self.oid, 'toggle_bookmark', [oid], {}
         )
-        return sorted([task.upper() for task in statues])
 
     def update_presets(self, filter_data=None, sort_data=None):
         if filter_data:
