@@ -208,6 +208,12 @@ class ClassicHome(flow.Object):
     )
 
 
+class Wizard(flow.Object):
+
+    def _fill_ui(self, ui):
+        ui["custom_page"] = "libreflow.custom_home.wizard.WizardHomePageWidget"
+
+
 class Home(flow.Object):
     """
     This Home is completly overrided bu the custom_page
@@ -215,6 +221,7 @@ class Home(flow.Object):
     """
 
     ClassicHome = flow.Child(ClassicHome)
+    Wizard = flow.Child(Wizard)
 
     def _fill_ui(self, ui):
         ui["custom_page"] = "libreflow.custom_home.ui.ProjectHomePageWidget"
