@@ -206,7 +206,7 @@ class CreateTrackedFileAction(baseflow.file.CreateTrackedFileAction):
             "Creating file %s.%s" % (name, self.file_format.get())
         )
 
-        self._files.add_tracked_file(name, self.file_format.get(), prefix + name)
+        self._files.create_file(name, extension=self.file_format.get(), base_name=prefix+name)
         self._files.touch()
 
 
