@@ -139,3 +139,118 @@ class PlaySequenceAction(GenericRunAction):
 class PlaySequenceActionFromShot(PlaySequenceAction):
 
     _sequence = flow.Parent(3)
+
+
+class PlaySequenceSettings(flow.Object):
+
+    filler_type = flow.Param([], SequenceFillerChoiceValue)
+    duration    = flow.IntParam(1)
+
+
+class TaskSelect(flow.values.ChoiceValue):
+
+    project_settings = flow.Parent(3)
+
+    def choices(self):
+        return self.project_settings.task_manager.default_tasks.mapped_names()
+    
+    def revert_to_default(self):
+        default_value = self.choices()[0]
+        self.set(default_value)
+
+
+class RemovePriorityFileAction(flow.Action):
+
+    ICON = ('icons.gui', 'remove-symbol')
+
+    _item = flow.Parent()
+    _map  = flow.Parent(2)
+
+    def needs_dialog(self):
+        return False
+    
+    def run(self, button):
+        map = self._map
+        map.remove(self._item.name())
+        map.touch()
+
+
+class CreatePriorityFileAction(flow.Action):
+
+    default_tasks = flow.Param([], TaskSelect)
+    file_name     = flow.Param('')
+    order         = flow.IntParam(0)
+
+    _map = flow.Parent()
+
+    def needs_dialog(self):
+        return True
+    
+    def get_buttons(self):
+        self.message.set('<h2>Create a priority file preset</h2>')
+        self.default_tasks.revert_to_default()
+        self.file_name.set('')
+        return ['Add', 'Cancel']
+    
+    def run(self, button):
+        if button == 'Cancel':
+            return
+        
+        i = 0
+        while self._map.has_mapped_name('priority%04i' % i):
+            i += 1
+        
+        priority_file = self._map.add('priority%04i' % i)
+        priority_file.task_name.set(self.default_tasks.get())
+        priority_file.file_name.set(self.file_name.get())
+        priority_file.order.set(self.order.get())
+
+        self._map.touch()
+
+
+class PriorityFile(flow.Object):
+
+    task_name   = flow.Param()
+    file_name   = flow.Param()
+    order       = flow.IntParam()
+
+    remove = flow.Child(RemovePriorityFileAction).ui(hidden=True)
+    
+    def get_icon(self):
+        name, ext = os.path.splitext(
+            self.file_name.get()
+        )
+        if ext:
+            return FILE_ICONS.get(
+                ext[1:], ('icons.gui', 'text-file-1')
+            )
+        else:
+            return ('icons.gui', 'folder-white-shape')
+
+
+class PriorityFilesMap(flow.Map):
+
+    add_priority_file = flow.Child(CreatePriorityFileAction).ui(label='Add')
+
+    @classmethod
+    def mapped_type(cls):
+        return PriorityFile
+    
+    def mapped_items(self, page_num=0, page_size=None):
+        names = self.mapped_names(page_num, page_size)
+        items = []
+        
+        for name in names:
+            items.append(self.get_mapped(name))
+      
+        return sorted(items, key=lambda item: item.order.get())
+    
+    def columns(self):
+        return ['Task', 'Name']
+    
+    def _fill_row_cells(self, row, item):
+        row['Task'] = item.task_name.get()
+        row['Name'] = item.file_name.get()
+    
+    def _fill_row_style(self, style, item, row):
+        style['Name_icon'] = item.get_icon()
